<?php
/**
 * @version     5.2.4
 * @package     com_gavoting
 * @copyright   Copyright (C) 2011. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Created by Glenn Arkell - http://www.glennarkell.com.au
 */

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Installer\Installer;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\Filesystem\Path;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\Mail\MailTemplate;
use \Joomla\CMS\MVC\Model\AdminModel;                                                
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\Installer\InstallerScript;
use \Joomla\CMS\Installer\Adapter\InstallerAdapter;
use \Joomla\CMS\Installer\Adapter\ComponentAdapter;
use \Joomla\CMS\Installer\Adapter\ModuleAdapter;
use \Joomla\CMS\Installer\Adapter\PluginAdapter;
use \Joomla\CMS\Filter\OutputFilter;

class com_gavotingInstallerScript extends InstallerScript
{

	/**
	 * The extension name. This should be set in the installer script.
	 */
	public $extName = 'Election Voting';

	public $compName = 'gavoting';

	public $mainView = 'positions';

	/**
	 * This version string. This should be set in the installer script.
	 */
	private $newRelease = '5.2.4';
	/**
	 * Minimum PHP version required to install the extension
	 */
	protected $minimumPhp = '8.0.13';
	/**
	 * Minimum Joomla! version required to install the extension
	 */
	protected $minimumJoomla = '5.0.0';
	/**
	 * Minimum MySQL version required to install the extension
	 */
	protected $minimumMySql = '5.7.14';
	/**
	 * List of required PHP extensions.
	 */
	protected $PHPextensions = array('dom', 'json', 'SimpleXML');
	/**
	 * @var  CMSApplication  Holds the application object
	 */
	private $app;
	/**
	 * @var  string  During an update, it will be populated with the old release version
	 */
	private $oldRelease;

	public $mailTmplSuffixs = array("electproxy","electnom","electnomsec","electvote");

	public $mailTags = array("sitename","link_text","name","principal","nominee","position","voter","seconder");
	
	/**
	 *  Constructor
	 */
	public function __construct()
	{
		$this->app = Factory::getApplication();
		//$this->minimumJoomla = $minimumJoomla;
		//$this->minimumPhp    = JOOMLA_MINIMUM_PHP;
        $this->gTours = array(
           'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_LBL'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_DESC',
           'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_POSITION_LBL'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_POSITION_DESC',
           'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_LBL'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_DESC',
           'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_LBL'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_DESC'
           );
	}

	/**
	 * @param   string  $parent  parent
	 * @return void
	 */
	public function discover_install($parent)
	{
		return self::install($parent);
	}

	/**
	 * method to run before an install/update/uninstall method
	 * @return void
	 */
	public function preflight($type, $parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_PREFLIGHT_'.STRTOUPPER($type).'_TEXT') . '</p>';

		if (JVERSION < $this->minimumJoomla) {
			$this->app->enqueueMessage(Text::sprintf('GA_INSTALL_CHECK_FAIL',$this->minimumJoomla,JVERSION), 'danger');
			return false;
		} else {
			$this->app->enqueueMessage(Text::sprintf('GA_INSTALL_CHECK_OK',$this->minimumJoomla,JVERSION), 'message');
			return parent::preflight($type, $parent);
		}

		if (STRTOUPPER($type) == 'INSTALL')
		{
			// check if requirements are met
			//$this->checkRequirements();
		}

		if (STRTOUPPER($type) == 'UPDATE')
		{
			// check if old 3.9 version of component originally installed
			$compvers = $this->getComponentVersion();
			if (substr($compvers,0,1) == 1 && substr(JVERSION,0,1) == 4) {
				$this->app->enqueueMessage(sprintf("J3 (%s) version of component installed - need to clean up.", $compvers), 'notice');
				//$this->removeOldFiles();
			} else {
				$this->app->enqueueMessage(sprintf("J4+ (%s) version of component installed.", $compvers), 'notice');
			}

		}

	}

	/**
	 * method to install the component
	 * @return void
	 */
	public function install($parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_GAVOTING_INSTALL_TEXT') . '</p>';

		/* --------------------------------  Action Log  ------------------------------ */
		// New action logging system
		//$extension = 'com_gavoting';
        $ActLog = $this->checkIfActionLog();
        if (!$ActLog) {
			$this->loadToActionLog();
			echo '<p>' . Text::_('Action Logging Setup') . '</p>';
		}
        $ALConf = $this->checkIfActionLogConfig();
        if (!$ALConf) {
			$this->loadToActionLogConfig('user_id', 'id', 'position_id', '#__gavoting_nominations','COM_GAVOTING_ACTION');
			echo '<p>' . Text::_('Action Log Configuration Setup') . '</p>';
		}

		/* --------------------------------  Dashboard  ------------------------------ */
		$dashB = $this->checkDashboard($this->compName);
		if (!$dashB) {
			$this->addDashboardMenu($this->compName, $this->compName);
		}

		/* --------------------------------  Mail Templates  ------------------------------ */
		// setup id for the template
        foreach ($this->mailTmplSuffixs as $tmpl) {
            $template_id = 'com_'.$this->compName.'.'.$tmpl;
            $tmplExists = $this->checkMailTemplates($template_id);
            if (!$tmplExists) {
                $this->addMailTemplate($tmpl, $this->mailTags);
            }
        }

	}

	/**
	 * method to update the component
	 * @return void
	 */
	public function update($parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_GAVOTING_UPDATE_TEXT') . '</p>';

		/* --------------------------------  Action Log  ------------------------------ */
		// New action logging system
        $ActLog = $this->checkIfActionLog();
        if (!$ActLog) {
			$this->loadToActionLog();
			echo '<p>' . Text::_('Action Logging Setup') . '</p>';
		}
        $ALConf = $this->checkIfActionLogConfig();
        if (!$ALConf) {
			$this->loadToActionLogConfig('user_id', 'id', 'position_id', '#__gavoting_nominations','COM_GAVOTING_ACTION');
			echo '<p>' . Text::_('Action Log Configuration Setup') . '</p>';
		}

		/* --------------------------------  Dashboard  ------------------------------ */
		$dashB = $this->checkDashboard($this->compName);
		if (!$dashB) {
			$this->addDashboardMenu($this->compName, $this->compName);
		}

		/* --------------------------------  Mail Templates  ------------------------------ */
		// setup id for the template
        foreach ($this->mailTmplSuffixs as $tmpl) {
            $template_id = 'com_'.$this->compName.'.'.$tmpl;
            $tmplExists = $this->checkMailTemplates($template_id);
            if (!$tmplExists) {
                $this->addMailTemplate($tmpl, $this->mailTags);
            }
        }

	}

	/**
	 * method to uninstall the component
	 * @return void
	 */
	public function uninstall($parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_UNINSTALL_TEXT') . '</p>';

		/* --------------------------------  Mail Templates  ------------------------------ */
        foreach ($this->mailTmplSuffixs as $tmpl) {
            $template_id = 'com_'.$this->compName.'.'.$tmpl;
            $tmplExists = $this->checkMailTemplates($template_id);
            if ($tmplExists) {
                $this->removeMailTemplate($template_id);
            }
        }

		/* --------------------------------  Dashboard  ------------------------------ */
		$dashMod = $this->checkDashboard($this->compName);
		if($dashMod && !$dashMod->id) {
            $this->removeDashboardMenu($this->compName);
            $this->deleteData('#__modules_menu', 'moduleid', '=', 0);
            $this->app->enqueueMessage('Deleted Module Menu', 'notice');
            $this->deleteData('#__assets', 'id', '=', $dashMod->asset_id);
            $this->app->enqueueMessage('Deleted Asset', 'notice');
            $this->deleteData('#__action_log_config', 'id', '=', 0);
            $this->app->enqueueMessage('Deleted Action Log', 'notice');
        }

		/* --------------------------------  Guided Tours  ------------------------------ */
        // if guided tours exist, remove them
        if (isset($this->gTours) && is_array($this->gTours) && !empty($this->gTours)) {
            foreach ($this->gTours as $gtUid => $title) {
                $tourUID =  STRTOLOWER($this->compName.'-'.$gtUid);
                $tourExists = $this->checkTourExists($tourUID);
                if ($tourExists) {
                    $this->removeTour($tourExists->id);
                }
            }
        }

	}

	/**
	 * method to run after an install/update/uninstall method
	 * @return void
	 */
	public function postflight($type, $parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_POSTFLIGHT_'.STRTOUPPER($type).'_TEXT') . '</p>';

		// setup data for first time install
        if (STRTOUPPER($type) == "INSTALL") {
            
            // Load categories
			$cattitles = array("2024");
            $cat_id = $this->createCategory($cattitles);
			$this->updatePositions($cat_id);
			
			// reinstall parameter to be passed
            $this->checkGuidedTours(false);

		}

		// setup data for first time install
        if (STRTOUPPER($type) == "UPDATE") {
            // update custom fields group setting of context - for 4.6.2
            if ($this->newRelease >= '4.6.2') {
                $this->updateCustomFields();
            }

            foreach ($this->mailTmplSuffixs as $tmpl) {
                $template_id = 'com_'.$this->compName.'.'.$tmpl;
                $tmplupds = $this->updateMailTemplates($template_id);
                if ($tmplupds) {
                    $this->app->enqueueMessage('Mail Templates Updated - '.$template_id, 'notice');
                }    
            }

    		// clean up old sql update file
			$path = Path::clean( JPATH_ADMINISTRATOR . '/components/com_' . $this->compName . '/sql/updates/mysql/');
			$this->deleteFiles($path, '0.0.01', '.sql');
			$this->deleteFiles($path, '1.3.00', '.sql');
			$this->deleteFiles($path, '1.3.01', '.sql');
			$this->deleteFiles($path, '1.5.02', '.sql');
			$this->deleteFiles($path, '1.6.00', '.sql');
			$this->deleteFiles($path, '4.7.0', '.sql');
			$this->deleteFiles($path, '4.2.2', '.sql');
			$this->deleteFiles($path, '4.5.1', '.sql');

			// reinstall parameter to be passed
            $this->checkGuidedTours(false);

		}


        // After install/update redirect to component admin default view
		//$parent->getParent()->setRedirectUrl('index.php?option=com_'.$this->compName);

		// Clear Joomla system cache.
		$cache = Factory::getCache();
		$cache->clean('_system');
	}

	/**
	 * *********************  Guided Tours Setup  *******************************
	 */

	/**
	 * Check for Guided Tours
	 * @param   boolean  true if reinstall is to be performed
	 */
	public function checkGuidedTours($reinstall)
	{
        // check for guided tours and create if required
        if (isset($this->gTours) && is_array($this->gTours) && !empty($this->gTours)) {
            foreach ($this->gTours as $gtUid => $desc) {
                $tourUID =  STRTOLOWER($this->compName.'-'.Text::_($gtUid));
                $tourExists = $this->checkTourExists($tourUID);

                if ($tourExists && $reinstall) {
                    // remove old GTs because changes made and we need to remove old steps
                    $this->removeTour($tourExists->id);
                    $tourExists = false;
                }

                if (!$tourExists) {
                    // set the start url and create tour
                    if ($gtUid == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_LBL') {
                        $url = 'administrator/index.php';
                    } elseif ($gtUid == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_LBL') {
                        $url = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
                    } elseif ($gtUid == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_POSITION_LBL') {
                        $url = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
                    } elseif ($gtUid == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_LBL') {
                        $url = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
                    } else {
                        continue;
                    }
                    $gtID = $this->createGuidedTour($gtUid, $desc, $url);
                    $this->app->enqueueMessage(Text::sprintf('COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_LOADED', Text::_($gtUid), $gtID), 'notice');
                    // create steps array
                    $recs = $this->setupGuidedTourSteps($gtUid);
                    // create tour steps
                    $this->createGuidedTourSteps($gtID, $recs);
                }
            }
        }

	}

	/**
	 * Check if a Guided Tour entry exists
	 * @param   string  $tourUid for the uid reference
	 * @return boolean or object
	 */
	public function checkTourExists($tourUid)
	{
        $result = false;
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' SELECT * FROM #__guidedtours WHERE extensions = '.$db->Quote('["com_'.$this->compName.'"]') . ' AND uid = '.$db->Quote($tourUid) );
		try {
		    $result = $db->loadObject();
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'danger');
		}

		return $result;
	}

	/**
	 * Create a Guided Tour record
	 * @param   string  $tourName for the uid reference and title
	 * @param   string  $url to set where the tour starts from
	 * @return boolean or record id
	 */
	public function createGuidedTour($tourName, $desc, $url)
	{
        $today = Factory::getDate()->toSql();
        $userId = Factory::getApplication()->getIdentity()->id;
        $db = Factory::getContainer()->get('DatabaseDriver');
        $gtour = new \stdClass();
        $gtour->title = $tourName;
        $gtour->uid = $this->compName.'-'.STRTOLOWER(Text::_($tourName));
        $gtour->description = $desc;
        $gtour->extensions = '["com_'.$this->compName.'"]';
        $gtour->url = $url;
        $gtour->created = $today;
        $gtour->created_by = $userId;
        $gtour->modified = $today;
        $gtour->modified_by = $userId;
        $gtour->language = '*';
        $gtour->published = 1;
        $gtour->note = '';
        $gtour->access = 1;
        if ($url == 'administrator/index.php') {
            $gtour->autostart = 1;
        }
		try {
		    $result = $db->insertObject('#__guidedtours', $gtour);
            return $db->insertid();
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'warning');
		    return false;
		}

	}

	/**
	 * Create a Guided Tour record
	 */
	public function createGuidedTourSteps($id, $recs)
	{
        $today = Factory::getDate()->toSQL();
        $userId = Factory::getApplication()->getIdentity()->id;
        foreach ($recs as $data) {
            $db = Factory::getContainer()->get('DatabaseDriver');
            $gtstep = new \stdClass();
            $gtstep->tour_id = $id;
            $gtstep->title = $data['title'];
            $gtstep->published = 1;
            $gtstep->description = $data['desc'];
            $gtstep->position = $data['position'];
            $gtstep->target = $data['target'];
            $gtstep->type = $data['type'];
            $gtstep->interactive_type = $data['intertype'];
            $gtstep->url = $data['url'];
            $gtstep->language = '*';
            $gtstep->created = $today;
            $gtstep->created_by = $userId;
            $gtstep->modified = $today;
            $gtstep->modified_by = $userId;
    		try {
    		    $result = $db->insertObject('#__guidedtour_steps', $gtstep);
    		} catch (RuntimeException $e) {
    		    $this->app->enqueueMessage($e->getMessage(), 'warning');
    		}
		}
        return true;

	}

	/**
	 * Removes the guided tour entries
	 * @param int $id The guided tour ID reference
	 * @return  void
	 */
	public function removeTour($id)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' DELETE FROM #__guidedtours WHERE id = '. (int) $id );
		try {
		    $db->execute();
			$this->app->enqueueMessage(Text::sprintf('COM_'.STRTOUPPER($this->compName).'_REMOVE_GUIDEDTOUR_SUCCESS', $id), 'notice');
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'danger');
		}

		$db1 = Factory::getContainer()->get('DatabaseDriver');
	    $db1->setQuery(' DELETE FROM #__guidedtour_steps WHERE tour_id = '. (int) $id );
		try {
		    $db1->execute();
			$this->app->enqueueMessage(Text::sprintf('COM_'.STRTOUPPER($this->compName).'_REMOVE_GUIDEDTOURSTEPS_SUCCESS', $id), 'notice');
		} catch (RuntimeException $e1) {
		    $this->app->enqueueMessage($e1->getMessage(), 'danger');
		}
	}

	/**
	 * Set up the step entries
	 * @param   string  $ref to identify which tour the steps belong to
	 *          types - 0 = Next, 1 = Redirect, 2 = Interactive
	 *          interactive types - 1 = Form Submit, 2 = Text Field, 4 = Button, 3 = Other
	 * @return boolean or array
	 */
	public function setupGuidedTourSteps($ref)
	{
        $returnURL = 'administrator/index.php?option=com_cpanel&view=cpanel&dashboard='.$this->compName;
        $recs = array();

        if ($ref == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_LBL') {
            $recs = $this->setupStdWelcomeSteps();
        }

        if ($ref == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_LBL') {
            $recs = $this->setupStdMenuSteps($returnURL);
        }

        if ($ref == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_POSITION_LBL') {
            // from the list screen, redirect to the dasboard
            $recs[] = $this->setupStdRedirectStep('', $returnURL);
            // link to the dasboard entry to see the list screen
            $recs[] = $this->setupStdListlinkStep('position');
            // standard new record button
            $recs[] = $this->setupStdNewStep('position');
            // standard fields
            $recs[] = $this->setupStdFieldStep('bottom', 'pos_name', 'position');
            $recs[] = $this->setupStdFieldStep('bottom', 'cat_id', 'position');
            $recs[] = $this->setupStdFieldStep('top', 'pos_qty', 'position');
            $recs[] = $this->setupStdFieldStep('top', 'elect_date', 'position');
            $recs[] = $this->setupStdFieldStep('top', 'elected', 'position');
            $recs[] = $this->setupStdFieldStep('top', 'jd_id', 'position');
            // extra tab link
            $recs[] = $this->setupStdTabStep('extrainfo');
            $recs[] = $this->setupStdFieldStep('bottom', 'comment', 'position');
            // end of process save & congrats
            $recs[] = $this->setupStdSaveCloseStep($returnURL);
            $recs[] = $this->setupStdCongratsStep($returnURL);

        }

        if ($ref == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_LBL') {
            // from the list screen, redirect to the dasboard
            $recs[] = $this->setupStdRedirectStep('', $returnURL);
            // link to the dasboard entry to see the list screen
            $recs[] = $this->setupStdListlinkStep('config');

            $recs[] = $this->setupStdFieldStep('bottom', 'dateFormat', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'agm_date', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'close_noms', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'open_votes', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'close_votes', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'ignore_prefix', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'jd_cat', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'limited_office', 'config');

            $recs[] = $this->setupStdTabStep('gavotingmembers');
            $recs[] = $this->setupStdFieldStep('bottom', 'excl_user_gp', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'restrict_users', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'restrict_nomees', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'restrict_nomtors', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'restrict_voters', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'bundle_region', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'bio_field', 'config');

            $recs[] = $this->setupStdTabStep('nominrules');
            $recs[] = $this->setupStdFieldStep('bottom', 'allow_anynom', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'auto_accept', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'disp_unaccept', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'show_votes', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'req_sec_nom', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'allow_sec_nom', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'extra_rules', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'show_nominfo', 'config');

            $recs[] = $this->setupStdTabStep('votingrules');
            $recs[] = $this->setupStdFieldStep('bottom', 'pref_voting', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'allow_proxy', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'reg_proxy', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'restrict_proxy', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'fwd_proxy', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'proxy_type', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'notif_proxy', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'multi_votes', 'config');

            $recs[] = $this->setupStdTabStep('gavotingmotions');
            $recs[] = $this->setupStdFieldStep('bottom', 'notag_colour', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'req_sec_mot', 'config');
            $recs[] = $this->setupStdFieldStep('top', 'allow_sec_mot', 'config');

            $recs[] = $this->setupStdTabStep('gavotingmultiuser');
            $recs[] = $this->setupStdFieldStep('bottom', 'multi_users', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'single_mship', 'config');

            $recs[] = $this->setupStdTabStep('integration');
            $recs[] = $this->setupStdFieldStep('bottom', 'test_comp', 'config');
            $recs[] = $this->setupStdFieldStep('bottom', 'tmpl_mail', 'config');

            $recs[] = $this->setupStdTabStep('permissions');
            $recs[] = $this->setupStdFieldStep('bottom', 'create', 'config');

            $recs[] = $this->setupStdSaveCloseStep($returnURL);
            $recs[] = $this->setupStdCongratsStep($returnURL);

        }

		return $recs;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @param   string $target
	 * @return  array $rec
	 */
	public function setupStdWelcomeSteps($target = '', $return = '')
	{
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_MENU_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_MENU_DESC',
              'position'=>'right',
              'target'=>'#sidebarmenu',
              'type'=>0,
              'intertype'=>2,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_DASHBOARD_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_DASHBOARD_DESC',
              'position'=>'right',
              'target'=>'.menu-dashboard a[href*="dashboard='.$this->compName.'"]',
              'type'=>2,
              'intertype'=>4,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_DESC',
              'position'=>'bottom',
              'target'=>'',
              'type'=>0,
              'intertype'=>1,
              'url'=>'#cpanel-modules a[href*="view='.$this->compName.'"]'
              );

		return $recs;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @param   string $target
	 * @return  array $rec
	 */
	public function setupStdRedirectStep($target = '', $return = '')
	{
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_REDIRECT_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_REDIRECT_DESC',
              'position'=>'top',
              'target'=>$target,
              'type'=>1,
              'intertype'=>2,
              'url'=>$return
              );

		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @param   string $list (singular of the list plurals)
	 * @return  array $rec
	 */
	public function setupStdListlinkStep($list = '')
	{
        if ($list == 'config') {
            $target = 'div.cpanel-modules.cpanel-'.$this->compName.' ul.list-group li.list-group-item a[href*="option=com_config&view=component&component=com_'.$this->compName.'"]';
            $url = 'administrator/index.php?option=com_config&view=component&component=com_'.$this->compName.'&path=&return=';
        } else {
            $target = 'div.cpanel-modules.cpanel-'.$this->compName.' ul.list-group li.list-group-item a[href*="option=com_'.$this->compName.'&view='.$list.'s"]';
            $url = 'administrator/index.php?option=com_'.$this->compName.'&view='.$list.'s';
        }
        // link to the dasboard entry to see the list screen
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_'.STRTOUPPER($list).'_LIST_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_'.STRTOUPPER($list).'_LIST_DESC',
              'position'=>'bottom',
              'target'=>$target,
              'type'=>2,
              'intertype'=>1,
              'url'=>$url
              );

		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @param   string $list (singular of the list plurals)
	 * @return  array $rec
	 */
	public function setupStdNewStep($list = '')
	{
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_'.STRTOUPPER($list).'_NEW_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_'.STRTOUPPER($list).'_NEW_DESC',
              'position'=>'bottom',
              'target'=>'.button-new',
              'type'=>2,
              'intertype'=>1,
              'url'=>''
              );

		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @param   string $list (singular of the list plurals)
	 * @return  array $rec
	 */
	public function setupStdFieldStep($position = 'bottom', $field = '', $list = '')
	{
        if ($field == 'create') {
            $target = '';
        } else {
            $target = '#jform_'.$field;
        }
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_'.STRTOUPPER($list).'_'.STRTOUPPER($field).'_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_'.STRTOUPPER($list).'_'.STRTOUPPER($field).'_DESC',
              'position'=>$position,
              'target'=>$target,
              'type'=>2,
              'intertype'=>2,
              'url'=>''
              );

		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @param   string $list
	 * @return  array $rec
	 */
	public function setupStdTabStep($tab = '')
	{
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_DESC',
              'position'=>'bottom',
              'target'=>'button[aria-controls='.$tab.']',
              'type'=>2,
              'intertype'=>4,
              'url'=>''
              );

		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @return  array $rec
	 */
	public function setupStdSaveCloseStep($return = '')
	{
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_DESC',
              'position'=>'bottom',
              'target'=>'#toolbar #toolbar-save button.button-save',
              'type'=>2,
              'intertype'=>1,
              'url'=>$return
              );

		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @return  array $rec
	 */
	public function setupStdCongratsStep($return = '')
	{
        $rec = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_DESC',
              'position'=>'bottom',
              'target'=>'',
              'type'=>1,
              'intertype'=>2,
              'url'=>$return
              );
		return $rec;
	}

	/**
	 * Setup standard guided tour steps
	 * @param   string $return URL
	 * @return  array $rec
	 */
	public function setupStdMenuSteps()
	{
        $returnURL = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
        $menuDash = 'administrator/index.php?option=com_cpanel&view=cpanel&dashboard=menus';
        $recs = array();
        //$recs[] = $this->setupStdRedirectStep('', $menuDash);

        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_MENU_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_WELCOME_MENU_DESC',
              'position'=>'right',
              'target'=>'#sidebarmenu',
              'type'=>0,
              'intertype'=>2,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_DASHBOARD_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_DASHBOARD_DESC',
              'position'=>'right',
              'target'=>'.menu-dashboard a[href*="dashboard=menus"]',
              'type'=>2,
              'intertype'=>4,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_NEW_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_NEW_DESC',
              'position'=>'left',
              'target'=>'#cpanel-modules .cpanel-menus .menu-quicktask a[href*="task=item.add&menutype=mainmenu"]',
              'type'=>2,
              'intertype'=>4,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_TITLE_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_TITLE_DESC',
              'position'=>'bottom',
              'target'=>'#jform_title',
              'type'=>2,
              'intertype'=>2,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_OWNINGMENU_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_OWNINGMENU_DESC',
              'position'=>'bottom',
              'target'=>'#myTab #details #jform_menutype',
              'type'=>2,
              'intertype'=>2,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_TYPE_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MENU_TYPE_DESC',
              'position'=>'bottom',
              'target'=>'#myTab #details #jform_type',
              'type'=>0,
              'intertype'=>2,
              'url'=>''
              );
        $recs[] = array(
              'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_LBL',
              'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_DESC',
              'position'=>'bottom',
              'target'=>'#toolbar #toolbar-dropdown-save-group #save-group-children-save button.button-save',
              'type'=>0,
              'intertype'=>1,
              'url'=>''
              );

		return $recs;
	}

	/**
	 * *********************  Dashborad stuff  *******************************
	 */
	/**
	 * Check if a Dashboard module entry exists
	 * @param   string $component Component name
	 * @return boolean or object
	 */
	public function checkDashboard($component = 0)
	{
		$result = false;
        if ($component) {
			$db = Factory::getContainer()->get('DatabaseDriver');
	        $db->setQuery(' SELECT * FROM #__modules WHERE position = '.$db->Quote('cpanel-'.$component) );
		    try {
		        $result = $db->loadObject();
		    } catch (RuntimeException $e) {
		        $this->app->enqueueMessage($e->getMessage(), 'danger');
		    }
	    }

		return $result;
	}

	/**
	 * Removes the dashboard menu module
	 * @param int $id The dashboard module id reference
	 * @return  void
	 */
	public function removeDashboardMenu($component)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' DELETE FROM #__modules WHERE position = '.$db->Quote('cpanel-'.$component) );
		try {
		    $db->execute();
			$this->app->enqueueMessage(Text::_('COM_'.STRTOUPPER($this->compName).'_REMOVE_DASHBOARD_SUCCESS'), 'notice');
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'danger');
		}
	}

	/**
	 * *********************  MailTemplate stuff  *******************************
	 */
	/**
	 * Update template records until core updated
	 */
	public function updateMailTemplates($template_id)
	{
		$table = '#__mail_templates';
        $db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('SELECT * FROM #__mail_templates WHERE '.$db->quotename('template_id').' = '.$db->quote($template_id));
		try {
		    $tmpl = $db->loadObject();
            if (!isset($tmpl->extension) || $tmpl->extension == '') {
                $params = json_decode($tmpl->params);
                $params->tags = $params->tags[0];
                $tmpl->params = json_encode($params);
                $tmpl->extension = 'com_'.$this->compName;
                $result = Factory::getContainer()->get('DatabaseDriver')->updateObject('#__mail_templates', $tmpl, 'template_id');
            }
            return true;
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'warning');
		    return false;
		}


	}

	/**
	 * Check if a MailTemplate entry exists
	 * @param   string $template_id (component + ext)
	 * @return boolean or object
	 */
	public function checkMailTemplates($template_id = 0)
	{
        $result = false;
		if ($template_id) {
			$db = Factory::getContainer()->get('DatabaseDriver');
	        $db->setQuery(' SELECT * FROM #__mail_templates WHERE template_id = '.$db->Quote($template_id) );
		    try {
		        $result = $db->loadObject();
		    } catch (RuntimeException $e) {
		        $this->app->enqueueMessage($e->getMessage(), 'danger');
		    }
	    }

		return $result;
	}

	/**
	 * Removes the mailTemplate records
	 * @param   string $template_id (component + ext)
	 * @return  void
	 */
	public function removeMailTemplate($template_id)
	{
		$this->deleteData('#__mail_templates', 'template_id', $template_id, '=');
		$this->app->enqueueMessage(Text::_('COM_'.STRTOUPPER($this->compName ?? '').'_EMAILTMPL_REMOVE_SUCCESS'), 'notice');
	}

	/**
	 * Create a new Mail Template
	 * @param   string $template_id (just the ext)
	 * @param   array $tags
	 * @return  void
	 */
	public function addMailTemplate($template_id, $tags)
	{
        $result = MailTemplate::createTemplate(
        	'com_'.$this->compName.'.'.$template_id,
        	'COM_'.STRTOUPPER($this->compName).'_EMAILTMPL_'.STRTOUPPER($template_id).'_SUBJECT',
        	'COM_'.STRTOUPPER($this->compName).'_EMAILTMPL_'.STRTOUPPER($template_id).'_BODY',
        	$tags,
        	'COM_'.STRTOUPPER($this->compName).'_EMAILTMPL_'.STRTOUPPER($template_id).'_HTMLBODY'
        );

		$this->app->enqueueMessage('Email Template Record Created . . . '.$template_id);

	}

	/**
	 * *********************  Data and Files stuff  *******************************
	 */
	/**
	 * Update data
	 * @param string $table Table name to be updated
	 * @param string $fieldname field name to be updated
	 * @param string $newvalue value to set the above fieldname
	 * @param string $wherefield field name of the where clause to test for
	 * @param string $oldvalue value of the field name in the where clause
	 * @param string $opatr operator for the test in the where clause
	 * @return  void
	 */
	public function updateData($table = '#__user_profiles', $wherefield = 'profile_key', $fieldname = 'profile_value', $oldvalue = 'Life', $newvalue = 'Life Member', $opatr = '=')
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('UPDATE '.$table.' SET '.$fieldname.' = '.$newvalue.' WHERE '.$wherefield.' '.$opatr.' '.$db->quote($oldvalue));
		$db->execute();
		$this->app->enqueueMessage('Records updated . . . '.$table, 'notice');

	}

	/**
	 * Delete old data
	 */
	public function deleteData($table = '#__extensions', $field = 'element', $value = '%com_weblinks%', $opatr = 'LIKE')
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('DELETE from '.$table.' WHERE '.$field.' '.$opatr.' '.$db->quote($value));
		$db->execute();
		$this->app->enqueueMessage('Records removed from . . . '.$table, 'notice');

	}

	/**
	 * Drop old tables
	 */
	public function dropTable($table = null)
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('DROP TABLE IF EXISTS '.$table);
		$db->execute();
		$this->app->enqueueMessage('Table dropped . . . '.$table, 'notice');

	}

	/**
	 * Create a folder
	 */
	public function createFolder($parent = null, $folder = null)
	{
        $path = Path::clean( JPATH_SITE . '/' . $parent . '/' . $folder );

		if (!is_dir($path) && !is_file($path)) {
            Folder::create($path);
			$this->app->enqueueMessage('Folder Created - '.$path, 'notice');
        } else {
			$this->app->enqueueMessage('Folder Already Exists - '.$path, 'notice');
		}
	}

	/**
	 * Delete unwanted file
	 */
	public function deleteFiles($path = null, $file = null, $ext = null)
	{
        $path_to_file = $path . $file . $ext;
		if (is_file($path_to_file)) {
			File::delete($path_to_file);
			$this->app->enqueueMessage('Deleted File - '.$path_to_file, 'notice');
		}
	}

	/**
	 * Delete unwanted folder
	 */
	public function deleteFolder($path = null)
	{
		if (is_dir($path)) {
			Folder::delete($path);
			$this->app->enqueueMessage('Deleted Folder - '.$path, 'notice');
		}
	}

	/**
	 * Get the version of the component
	 * @return version element of manifest
	 */
	public static function getComponentVersion()
	{
		$componentXML = Installer::parseXMLInstallFile(Path::clean(JPATH_ADMINISTRATOR . '/components/com_'.$this->compName.'/'.$this->compName.'.xml'));
        
		$version = $componentXML && isset($componentXML['version']) ? $componentXML['version'] : '5.0.0';

		return $version;
	}

	/**
	 * Check if a Dashboard module entry exists
	 * @param   string $component Component name
	 * @return boolean or object
	 */
	public function updateCustomFields()
	{
        $result = false;
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' UPDATE #__fields_groups SET context = '.$db->Quote('com_'.$this->compName).' WHERE context = '.$db->Quote('com_'.$this->compName.'.position') );
		try {
		    $db->execute();
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'danger');
		}
	}

	/**
	 * @param   string  $group    group
	 * @param   string  $element  element
	 * @return boolean
	 */
	public function enablePlugin($group, $element)
	{
		$plugin = Table::getInstance('extension');
		if (!$plugin->load(array('type' => 'plugin', 'folder' => $group, 'element' => $element)))
		{
			return false;
		}
		$plugin->enabled = 1;
		return $plugin->store();
	}

	/**
	 * Check versions of components
	 * @return boolean|integer
	 */
	public function checkRequirements()
	{
		$db   = Factory::getContainer()->get('DatabaseDriver');
		$pass = $this->checkVersion('PHP', $this->getCleanPhpVersion());
		$pass &= $this->checkVersion('Joomla!', JVERSION);
		//$pass &= $this->checkVersion('MySQL', $this->minimumMySql);
		$pass &= $this->checkVersion('MySQL', $db->getVersion());
		$pass &= $this->checkDbo($db->name, array('mysql', 'mysqli'));
		//$pass &= $this->checkExtensions($this->PHPextensions);
		return $pass;
	}

	/**
	 *  On some hosting the PHP version given with the version of the packet in the distribution
	 * @return string
	 */
	protected function getCleanPhpVersion()
	{
		$version = PHP_MAJOR_VERSION . '.' . PHP_MINOR_VERSION . '.' . PHP_RELEASE_VERSION;
		return $version;
	}

	/**
	 * @param   string  $name     name of the element
	 * @param   string  $version  version of the element
	 * @return boolean
	 * @throws Exception
	 */
	protected function checkJoomlaVersion($name, $version)
	{
		$app = Factory::getApplication();
		$major = 4;
		$minor = 0;
		foreach ($this->versions[$name] as $major => $minor)
		{
			if (!$major || version_compare($version, $major, '<'))
			{
				continue;
			}
			if (version_compare($version, $minor, '>='))
			{
				return true;
			}
			break;
		}
		if (!$major)
		{
			$minor = reset($this->versions[$name]);
		}
		$recommended = end($this->versions[$name]);
		$app->enqueueMessage(sprintf("%s %s is not supported. Minimum required version is %s %s, but it is highly recommended to use %s %s or later.",
			$name, $version, $name, $minor, $name, $recommended
		), 'notice'
		);
		return false;
	}

	/**
	 * @param   string  $name     name of the element
	 * @param   string  $version  version of the element
	 * @return boolean
	 * @throws Exception
	 */
	protected function checkVersion($name, $version)
	{
		$minor = $name == 'MySQL' ? $this->minimumMySql : 0;
		$app = Factory::getApplication();
		$major = $minor = 0;
		foreach ($this->versions[$name] as $major => $minor)
		{
			if (!$major || version_compare($version, $major, '<'))
			{
				continue;
			}
			if (version_compare($version, $minor, '>='))
			{
				return true;
			}
			break;
		}
		if (!$major)
		{
			$minor = reset($this->versions[$name]);
		}
		$recommended = end($this->versions[$name]);
		$app->enqueueMessage(sprintf("%s %s is not supported. Minimum required version is %s %s, but it is highly recommended to use %s %s or later.",
			$name, $version, $name, $minor, $name, $recommended
		), 'notice'
		);
		return false;
	}

	/**
	 * @param   string  $name   name
	 * @param   array   $types  types
	 * @return boolean
	 * @throws Exception
	 */
	protected function checkDbo($name, $types)
	{
		$app = Factory::getApplication();
		if (in_array($name, $types))
		{
			return true;
		}
		$app->enqueueMessage(sprintf("Database driver '%s' is not supported. Please use MySQL instead.", $name), 'notice');
		return false;
	}

	/**
	 * @param   array  $extensions  extensions
	 * @return integer
	 * @throws Exception
	 */
	protected function checkExtensions($PHPextensions)
	{
		$app = Factory::getApplication();
		$pass = 1;
		foreach ($PHPextensions as $name)
		{
			if (!extension_loaded($name))
			{
				$pass = 0;
				$app->enqueueMessage(sprintf("Required PHP extension '%s' is missing. Please install it into your system.", $name), 'notice');
			}
		}
		return $pass;
	}

	/**
	 * Update position records with category id on initial install
	 * @param   int $id   Category ID reference
	 * @return boolean
	 */
	public function updatePositions($cat_id = 0)
	{
        if ($cat_id) {
			$db = Factory::getContainer()->get('DatabaseDriver');
	        $db->setQuery(' UPDATE #__gavoting_positions SET cat_id = '.(int) $cat_id );
		    try {
		        $db->execute();
		    } catch (RuntimeException $e) {
		        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
		        return false;
		    }
	    }

		return true;
	}

    /**
    * Function to create category records
    * @param array category titles
    * @param string category group or type
    * @return void
    */
    public function createCategory($cat_titles)
    {
		foreach ($cat_titles as $cat) {
            $category = Table::getInstance('Category');
            $category->extension = 'com_'.$this->compName;
            $category->title = $cat;
            $category->alias = OutputFilter::stringUrlSafe($cat);
            $category->description = '';
            $category->published = 1;
            $category->access = 1;
            $category->params = '{"category_layout":"","image":"","image_alt":""}';
            $category->metadata = '{"page_title":"","author":"","robots":""}';
            $category->language = '*';
            // Set the location in the tree
            $category->setLocation(1, 'last-child');
            // Check to make sure our data is valid
            if (!$category->check()) {
                throw new \Exception(500, $category->getError());
                return false;
            }
            // Now store the category
            if (!$category->store(true)) {
                throw new \Exception(500, $category->getError());
                return false;
            }
	 	}
        // Build the path for our category
        $category->rebuildPath($category->id);
        echo '<p>' . Text::_('Categories created') . '</p>';
        return $category->id;
	}

 	/**
	 * **********  Action Log setup and checking ********************
	 */

	/**
	 * Check if extension is set in Action Logs register
	 * @param   string $extension   Extension name
	 * @return boolean
	 */
	public function checkIfActionLog()
	{
		$result = false;

		$db = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true)
			->select($db->quoteName('id'))
			->from($db->quoteName('#__action_logs_extensions'))
			->where($db->quoteName('extension') .' = '. $db->Quote($this->extension));
		$db->setQuery($query);
	    try {
	        // If it fails, it will throw a RuntimeException
	        $result = $db->loadResult();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage());
	    }

		return $result;
	}

	/**
	 * Insert extension to the Action Logs register
	 * @param   string $extension   Extension name
	 * @return boolean
	 */
	public function loadToActionLog()
	{
		$result = false;
        $db = Factory::getContainer()->get('DatabaseDriver');
        $db->setQuery(' INSERT into #__action_logs_extensions (extension) VALUES ('.$db->Quote($this->extension).') ' );
	    try {
	        // If it fails, it will throw a RuntimeException
	        $result = $db->execute();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage());
	        return false;
	    }

		return $result;
	}

	/**
	 * Check if extension is set in Action Logs register
	 * @param   string $extension   Extension name
	 * @return boolean
	 */
	public function checkIfActionLogConfig()
	{
		$result = false;

		$db = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true)
			->select($db->quoteName('id'))
			->from($db->quoteName('#__action_log_config'))
			->where($db->quoteName('type_alias') .' = '. $db->Quote($this->extension));
		$db->setQuery($query);
	    try {
	        // If it fails, it will throw a RuntimeException
	        $result = $db->loadResult();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage());
	    }

		return $result;
	}

	/**
	 * Insert extension to the Action Log Configuration record
	 * @param   string $extension   Extension name
	 * @return boolean
	 */
	public function loadToActionLogConfig($type, $key = 'id', $title = '', $tablename = '', $txtpref = '')
	{
		// Create and populate an object.
		$logConf = new stdClass();
		//$logConf->id = 0;
		$logConf->type_title = $type;
		$logConf->type_alias = $this->extension;
		$logConf->id_holder = $key;
		$logConf->title_holder = $title;
		$logConf->table_name = $tablename;
		$logConf->text_prefix = $txtpref;

	    try {
	        // If it fails, it will throw a RuntimeException
			// Insert the object into the user profile table.
			$result = Factory::getContainer()->get('DatabaseDriver')->insertObject('#__action_log_config', $logConf);
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage());
	        return false;
	    }

		return $result;
	}

 	/**
	 * **********  Remove all old J3 version files in the old structure ********************
	 */

	/**
	 * Batch update of old J3 version of software
	 */
	function removeOldFiles()
	{
		$component = 'com_gavoting';
		$path = Path::clean( JPATH_SITE . '/components/' . $component . '/');
		$this->deleteFiles($path, 'index', '.html');
		$this->deleteFiles($path, 'gavoting', '.php');
		$this->deleteFiles($path, 'router', '.php');
		$this->deleteFiles($path, 'controller', '.php');

		$adpath = Path::clean( JPATH_ADMINISTRATOR . '/components/' . $component . '/');
		$this->deleteFiles($adpath, 'index', '.html');
		$this->deleteFiles($adpath, 'gavoting', '.php');
		$this->deleteFiles($adpath, 'access', '.xml');
		$this->deleteFiles($adpath, 'config', '.xml');
		$this->deleteFiles($adpath, 'controller', '.php');

        $folders = array(
			'controllers',
			'helpers',
			'models',
			'tables',
			'views'
			);
        $files = array(
			'motion',
			'nomination',
			'position',
			'voter'
			);

        // remove old folders and files - SITE
        foreach ($folders as $folder) {

			$path = Path::clean( JPATH_SITE . '/components/' . $component . '/' . $folder .'/');
			$adpath = Path::clean( JPATH_ADMINISTRATOR . '/components/' . $component . '/' . $folder .'/');

			if ($folder == 'controllers') {
				foreach ($files as $file) {
					$this->deleteFiles($path, 'index', '.html');
					$this->deleteFiles($adpath, 'index', '.html');
					$this->deleteFiles($path, $file, '.php');
					$this->deleteFiles($path, $file, 's.php');
					$this->deleteFiles($path, $file, 'form.php');
					$this->deleteFiles($path, $file, '.xml');
					$this->deleteFiles($adpath, $file, '.php');
					$this->deleteFiles($adpath, $file, 's.php');
				}
				$this->deleteFolders($path);
				$this->deleteFolders($adpath);
			}

			if ($folder == 'models') {
				foreach ($files as $file) {
					$this->deleteFiles($path, 'index', '.html');
					$this->deleteFiles($adpath, 'index', '.html');
					$this->deleteFiles($path, $file, '.php');
					$this->deleteFiles($path, $file, 's.php');
					$this->deleteFiles($path, $file, 'form.php');
					$this->deleteFiles($path, $file, '.xml');
					$this->deleteFiles($adpath, $file, '.php');
					$this->deleteFiles($adpath, $file, 's.php');

					$pathfield = $path . 'fields/';
					$adpathfield = $adpath . 'fields/';
					$this->deleteFiles($pathfield, 'index', '.html');
					$this->deleteFiles($adpathfield, 'index', '.html');
					$this->deleteFiles($adpathfield, 'createdby', '.php');
					$this->deleteFiles($adpathfield, 'customfield', '.php');
					$this->deleteFiles($adpathfield, 'filemultiple', '.php');
					$this->deleteFiles($adpathfield, 'members', '.php');
					$this->deleteFiles($adpathfield, 'modifiedby', '.php');
					$this->deleteFiles($adpathfield, 'nominees', '.php');
					$this->deleteFiles($adpathfield, 'positions', '.php');
					$this->deleteFiles($adpathfield, 'proffields', '.php');
					$this->deleteFiles($adpathfield, 'regions', '.php');
					$this->deleteFiles($adpathfield, 'submit', '.php');
					$this->deleteFiles($adpathfield, 'timecreated', '.php');
					$this->deleteFiles($adpathfield, 'timeupdated', '.php');

					$pathform = $path . 'forms/';
					$adpathform = $adpath . 'forms/';
					$this->deleteFiles($pathform, 'index', '.html');
					$this->deleteFiles($adpathform, 'index', '.html');
					$this->deleteFiles($pathform, 'filter_'.$file, 's.xml');
					$this->deleteFiles($pathform, $file, 'form.xml');
					$this->deleteFiles($adpathform, 'filter_'.$file, 's.xml');
					$this->deleteFiles($adpathform, $file, '.xml');
				}
				$this->deleteFolders($pathfield);
				$this->deleteFolders($pathform);
				$this->deleteFolders($path);
				$this->deleteFolders($adpathfield);
				$this->deleteFolders($adpathform);
				$this->deleteFolders($adpath);
			}

			if ($folder == 'views') {
				foreach ($files as $file) {
					$this->deleteFiles($path, 'index', '.html');
					$this->deleteFiles($adpath, 'index', '.html');
					// single
					$this->deleteFiles($path . $file . '/', 'view.html.php');
					$this->deleteFiles($path . $file . '/', 'index.html');
					$this->deleteFiles($path . $file . '/tmpl/', 'default.php');
					$this->deleteFiles($path . $file . '/tmpl/', 'index.html');
					$this->deleteFiles($path . $file . '/tmpl/', 'default.xml');
					$this->deleteFolders($path . $file . '/tmpl/');
					$this->deleteFolders($path . $file . '/');

					// form
					$this->deleteFiles($path . $file . 'form/', 'view.html.php');
					$this->deleteFiles($path . $file . 'form/', 'index.html');
					$this->deleteFiles($path . $file . 'form/tmpl/', 'default.php');
					$this->deleteFiles($path . $file . 'form/tmpl/', 'index.html');
					$this->deleteFiles($path . $file . 'form/tmpl/', 'default.xml');
					$this->deleteFolders($path . $file . 'form/tmpl/');
					$this->deleteFolders($path . $file . 'form/');

					// multiple
					$this->deleteFiles($path . $file . 's/', 'view.html.php');
					$this->deleteFiles($path . $file . 's/', 'index.html');
					$this->deleteFiles($path . $file . 's/tmpl/', 'default.php');
					$this->deleteFiles($path . $file . 's/tmpl/', 'default_filter.php');
					$this->deleteFiles($path . $file . 's/tmpl/', 'index.html');
					$this->deleteFiles($path . $file . 's/tmpl/', 'default.xml');
					$this->deleteFolders($path . $file . 's/tmpl/');
					$this->deleteFolders($path . $file . 's/');
				}
				$this->deleteFolders($path);
			}

			if ($folder == 'helpers') {
				$this->deleteFiles($path, 'index', '.html');
				$this->deleteFolders($path);
				$this->deleteFiles($adpath, 'index', '.html');
				$this->deleteFiles($adpath, 'gaupdparams', '.php');
				$this->deleteFiles($adpath, 'gavoting', '.php');
				$this->deleteFiles($adpath, 'listhelper', '.php');
				$this->deleteFolders($adpath);
			}

			if ($folder == 'tables') {
				$this->deleteFiles($adpath, 'index', '.html');
				foreach ($files as $file) {
					$this->deleteFiles($adpath, $file, '.php');
				}
				$this->deleteFolders($adpath);
			}
        }

		return true;
	}

}
